package aepClient

import (
	"context"
	"log"
	"strings"
	"time"

	"github.com/apache/pulsar-client-go/pulsar"
)

type MessageListener interface {
	OnMessage(msg []byte) error
}

var needStop bool = false
var consumer pulsar.Consumer = nil
var client pulsar.Client = nil

//开始消息接收
func Start(mqServer string, tenantId string, token string, topics []string, listener MessageListener) {
	var serverAddr string
	if strings.HasPrefix(mqServer, "pulsar://") || strings.HasPrefix(mqServer, "pulsar+ssl://") {
		serverAddr = mqServer
	} else {
		serverAddr = "pulsar+ssl://" + mqServer
	}

	var fullPathTopics []string = make([]string, len(topics))

	for index, topic := range topics {
		fullPathTopics[index] = "aep-msgpush/" + tenantId + "/" + topic
	}

	var err error
	client, err = pulsar.NewClient(pulsar.ClientOptions{
		URL:               serverAddr,
		OperationTimeout:  30 * time.Second,
		ConnectionTimeout: 30 * time.Second,
		Authentication:    pulsar.NewAuthenticationToken(token),
	})
	if err != nil {
		log.Fatalf("Could not instantiate Pulsar client: %v", err)
	}

	consumer, err = client.Subscribe(pulsar.ConsumerOptions{
		Topics:           fullPathTopics,
		SubscriptionName: tenantId,
		Type:             pulsar.Shared,
	})
	if err != nil {
		log.Fatal(err)
	}
	//defer consumer.Close()

	for !needStop {
		msg, err := consumer.Receive(context.Background())
		if err != nil {
			log.Println(err)
		} else {
			err := listener.OnMessage(msg.Payload())
			if err == nil {
				consumer.Ack(msg)
			}
		}
	}

	log.Println("mq consumer exit")
}

//停止消息接收
func Stop() {
	needStop = true
	if consumer != nil {
		consumer.Close()
	}

	if client != nil {
		client.Close()
	}
}
